=== modified file 'bzrlib/tests/test_uncommit.py'
--- bzrlib/tests/test_uncommit.py	2011-08-22 10:44:29 +0000
+++ bzrlib/tests/test_uncommit.py	2011-12-16 19:41:30 +0000
@@ -110,6 +110,27 @@
             "pointsatexisting": history[0]
             }, tree.branch.tags.get_tag_dict())
 
+    def test_uncommit_remove_tags_keeps_pending_merges(self):
+        tree, history = self.make_linear_tree()
+        copy = tree.bzrdir.sprout('copyoftree').open_workingtree()
+        copy.commit(message='merged', rev_id='merged')
+        tree.merge_from_branch(copy.branch)
+        tree.branch.tags.set_tag('pointsatmerged', 'merged')
+        history.append(tree.commit('merge'))
+        self.assertEquals('merged', tree.branch.tags.lookup_tag('pointsatmerged'))
+        self.assertEqual(history[2], tree.last_revision())
+        self.assertEqual((3, history[2]), tree.branch.last_revision_info())
+        tree.branch.tags.set_tag(u"pointsatexisting", history[1])
+        tree.branch.tags.set_tag(u"pointsatremoved", history[2])
+        uncommit.uncommit(tree.branch, tree=tree)
+        self.assertEqual(history[1], tree.last_revision())
+        self.assertEqual((2, history[1]), tree.branch.last_revision_info())
+        self.assertEquals([history[1], 'merged'], tree.get_parent_ids())
+        self.assertEqual({
+            "pointsatexisting": history[1],
+            "pointsatmerged": 'merged',
+            }, tree.branch.tags.get_tag_dict())
+
     def test_uncommit_keep_tags(self):
         tree, history = self.make_linear_tree()
         self.assertEqual(history[1], tree.last_revision())

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2011-08-22 10:53:21 +0000
+++ bzrlib/uncommit.py	2011-12-16 19:41:30 +0000
@@ -26,17 +26,17 @@
 from bzrlib.errors import BoundBranchOutOfDate
 
 
-def remove_tags(branch, graph, old_tip, new_tip):
+def remove_tags(branch, graph, old_tip, parents):
     """Remove tags on revisions between old_tip and new_tip.
 
     :param branch: Branch to remove tags from
     :param graph: Graph object for branch repository
     :param old_tip: Old branch tip
-    :param new_tip: New branch tip
+    :param parents: New parents
     :return: Names of the removed tags
     """
     reverse_tags = branch.tags.get_reverse_tag_dict()
-    ancestors = graph.find_unique_ancestors(old_tip, [new_tip])
+    ancestors = graph.find_unique_ancestors(old_tip, parents)
     removed_tags = []
     for revid, tags in reverse_tags.iteritems():
         if not revid in ancestors:
@@ -128,15 +128,15 @@
                     hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
                      hook_new_tip)
-            if branch.supports_tags() and not keep_tags:
-                remove_tags(branch, graph, old_tip, new_revision_id)
+            if not _mod_revision.is_null(new_revision_id):
+                parents = [new_revision_id]
+            else:
+                parents = []
             if tree is not None:
-                if not _mod_revision.is_null(new_revision_id):
-                    parents = [new_revision_id]
-                else:
-                    parents = []
                 parents.extend(reversed(pending_merges))
                 tree.set_parent_ids(parents)
+            if branch.supports_tags() and not keep_tags:
+                remove_tags(branch, graph, old_tip, parents)
     finally:
         for item in reversed(unlockable):
             item.unlock()

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2011-12-15 14:47:22 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2011-12-16 19:41:30 +0000
@@ -70,6 +70,9 @@
 * Report mistake trying to move a removed file with a non-ascii name without
   UnicodeEncodeError being raised. (Martin Packman, #898541)
 
+* Uncommit no longer removes tags if they are part of the working
+  trees pending merges. (Jelmer Vernooij, #905462)
+
 Documentation
 *************
 

